#ifndef _NODE_HANDLERS_H_
#define _NODE_HANDLERS_H_

class Task;
class SyntaxTreeNode;
class Token;

class NodeHandler {

public:
	virtual ~NodeHandler();

	virtual bool isValid(SyntaxTreeNode *node) = 0;
	virtual Token addToModel(Task &task, SyntaxTreeNode *node) = 0;
};

class MiddlePointNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);
};

class AngleNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);
};

class DistanceNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);
};

class TriangleNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);
};

class TriangleAreaNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class EqualNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class TimesNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class FindDistanceNodeHandler : public NodeHandler {

public: 
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class FindAngleNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class FindTriangleAreaNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class CosNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class CollinearPointsNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class FindTrapeziumAreaNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class PointInSegmentNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class ParallelLinesNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

class PointDivideSegmentNodeHandler : public NodeHandler {

public:
	virtual bool isValid(SyntaxTreeNode *node);
	virtual Token addToModel(Task &task, SyntaxTreeNode *node);

};

#endif // _NODE_HANDLERS_H_
