#ifndef _TASK_H_
#define _TASK_H_

#include <vector>
#include <map>
#include <iostream>
#include <string>

#include "polynomtask.h"

class Point;
class Condition;
class Question;

class Task {

public:
	Task();
	~Task();

	std::string getAvailablePointName();
	const Point &getPoint(const std::string &name);

	void addTriangleCondition(const Point &A, const Point &B, const Point &C);

	void addDistanceCondition(const Point &A, const Point &B, const std::string &distVar);
	void addDistanceToSegmentCondition(const Point &P, const Point &A, const Point &B, const std::string &distVar);
	void addMiddlePointCondition(const Point &A, const Point &B, const Point &M);
	void addAngleCondition(const Point &left, const Point &vertex, const Point &right, const std::string &angleVar);
	void addTriangleAreaCondition(const Point &first, const Point &second, const Point &third, const std::string &areaVar);

	void addDistanceQuestion(const Point &A, const Point &B);
	void addAngleQuestion(const Point &left, const Point &vertex, const Point &right);
	void addTriangleAreaQuestion(const Point &first, const Point &second, const Point &third);

	void addEqualCondition(const std::string &leftVar, const std::string &rightVar);
	void addTimesCondition(const std::string &leftVar, const std::string &rightVar, const std::string &resultVar);
	void addCollinearPointsCondition(const Point &first, const Point &second, const Point &third);

	void addTrapeziumAreaQuestion(const Point &A, const Point &B, const Point &C, const Point &D);
	void addPointInSegmentCondition(const Point &P, const Point &A, const Point &B);
	void addParallelLinesCondition(const Point &A, const Point &B, const Point &C, const Point &D);
	void addPointDivideSegmentCondition(const Point &point, const Point &left, const Point &right, const std::string &coefVariable);

	std::string getAvailableVariableName();

	std::vector<std::pair<GMPint, GMPint> > solve();

public:
	friend std::ostream &operator<<(std::ostream &out, const Task &task);

private:
	std::map<std::string, Point> myPoints;
	std::vector<Condition *> myConditions;
	std::vector<Question *> myQuestions;

	const std::string myAvailablePointNamePrefix;
	size_t myAvailablePointNumber;

	PolynomTask myPolynomTask;

private:
	std::string getCurrentAvailablePointName();
};

#endif // _TASK_H_
