#ifndef _TOKENIZER_H_
#define _TOKENIZER_H_

#include <iostream>
#include <string>
#include <deque>

class Token {

public:
	enum TokenType { ttIdentifier, ttNumber, ttDelimiter, ttError };

public:
	Token(const std::string &value, TokenType type, const std::pair<size_t, size_t> position);

	const std::string &getValue() const;
	TokenType getType() const;
	std::pair<int, int> getPosition() const;

	void print() const;

private:
	std::string myValue;
	TokenType myType;
	std::pair<size_t, size_t> myPosition;
};

class Tokenizer {

public:
	Tokenizer(std::istream &inputStream);

	Token nextToken();
	bool hasTokens() const;

private:
	enum TokenizerState { tsStart, tsIdentifier, tsNumber, tsSign, tsBeginComment, tsWaitForStar, tsEndComment };

private:
	std::istream &myInputStream;
	std::deque<Token> myTokens;

private:
	Tokenizer(const Tokenizer &);
	const Tokenizer &operator=(const Tokenizer &);

private:
	bool isDelimiter(char c) const;
	void scan();
};

#endif // _TOKENIZER_H_
