#ifndef POLYNOM_IMPL_H_
#define POLYNOM_IMPL_H_
#include "Polynom.h"
#include <stdio.h>
#include <algorithm>
#include <sstream>
#include <iostream>
#include <gmpxx.h>

inline int gcd(int a, int b) {
  if (a * b < 0) a = -a;
  while( 1 ) {
    a = a % b;
    if( a == 0 ) return b;
    b = b % a;
    if( b == 0 ) return a;
  }
}

template<class Map>
void MapAdd(Map& to, const Map& from) {
  if (from.empty()) {
    return;
  }
  if (to.empty()) {
    to = from;
    return;
  }
  typename Map::iterator it = to.begin();
  typename Map::const_iterator that = from.begin();
  typename Map::iterator toend = to.end();
  typename Map::const_iterator fromend = from.end();
  typename Map::iterator it1;
  typename Map::key_compare Comp;
  for (; that != fromend; ++that) {
    while ((it != toend) && Comp(it->first, that->first)) {
      ++it;
    }
    if (it != toend && (!Comp(that->first, it->first))) {
      it->second += that->second;
      if (it->second == 0) {
        it1 = it;
        ++it;
        to.erase(it1);
      }
    } else {
      if (it == to.begin()) {
        to.insert(*that);
      } else {
        it1 = it;
        --it1;
        to.insert(it1, *that);
      }
    }
  }
}

template<class Map>
void MapSub(Map& to, const Map& from) {
  if (from.empty()) {
    return;
  }
  if (to.empty()) {
    to = from;
    return;
  }
  typename Map::iterator it = to.begin();
  typename Map::const_iterator that = from.begin();
  typename Map::iterator toend = to.end();
  typename Map::const_iterator fromend = from.end();
  typename Map::iterator it1;
  typename Map::key_compare Comp;
  for (; that != fromend; ++that) {
    while ((it != toend) && Comp(it->first, that->first)) {
      ++it;
    }
    if (it != toend && (!Comp(that->first, it->first))) {
      it->second -= that->second;
      if (it->second == 0) {
        it1 = it;
        ++it;
        to.erase(it1);
      }
    } else {
      if (it == to.begin()) {
        it1 = to.insert(*that).first;
      } else {
        it1 = it;
        --it1;
        it1 = to.insert(it1, *that);
      }
      it1->second = -it1->second;
    }
  }
}

//-------------------Monom

inline Monom::Monom(const Monom& m) {
  myPowers = m.myPowers;
  myPower = m.myPower;
}

inline Monom::Monom() {
  myPower = 0;
}

inline Monom& Monom::operator=(const Monom& m) {
  myPowers = m.myPowers;
  myPower = m.myPower;
  return *this;
}

inline size_t& Monom::operator[](size_t n) {
  myPower = -1;
  return myPowers[n];
}

inline Monom Monom::operator*(const Monom& m) const {
  Monom temp = *this;
  temp*=m;
  return temp;
}

inline Monom Monom::operator/(const Monom& m) const {
  Monom temp = *this;
  temp/=m;
  return temp;
}

inline int Monom::getPower() const {
  if (myPower != -1) {
    return myPower;
  }
  (*(int*)&myPower) = 0;
  Monom* Me = (Monom*) this;
  std::map<size_t, size_t>::const_iterator iend = Me->myPowers.end();
  for (std::map<size_t, size_t>::iterator it = Me->myPowers.begin(); it != iend; ++it) {
    Me->myPower += it->second;
  }
  return myPower;
}

inline bool Monom::operator==(const Monom& m) const {
  if (getPower() != m.getPower()) {return false;}
  std::map<size_t, size_t>::const_iterator it;
  std::map<size_t, size_t>::const_iterator that = m.myPowers.begin();
  std::map<size_t, size_t>::const_iterator iend = myPowers.end();
  std::map<size_t, size_t>::const_iterator tend = m.myPowers.end();
  for (it = myPowers.begin(); (it != iend) && (that != tend); ++it) {
    if (it->first != that->first) return false;
    if (it->second != that->second) return false;
    ++that;
  }
  if (it != iend) return false;
  if (that != tend) return false;
  return true;
}

inline bool Monom::divisible(const Monom& m) const{
  if (getPower() < m.getPower()) {return false;}
  std::map<size_t, size_t>::const_iterator it = myPowers.begin();
  std::map<size_t, size_t>::const_iterator that = m.myPowers.begin();
  std::map<size_t, size_t>::const_iterator iend = myPowers.end();
  std::map<size_t, size_t>::const_iterator tend = m.myPowers.end();
  while ((it != iend) && (that != tend)) {
    if (that->first < it->first) {return false;}
    if (it->first == that->first) {
      if (it->second < that->second) {
        return false; 
      } else {
        ++that;
      }
    }
  ++it;
  }
  if (that != tend) {return false;}
  return true;
}

inline Monom& Monom::operator*=(const Monom& m) {
  MapAdd(myPowers, m.myPowers);
  if (myPower != -1) myPower += m.myPower;
  return *this;
}

inline Monom& Monom::operator/=(const Monom& m) {
  if (!divisible(m)) return *this;
  MapSub(myPowers, m.myPowers);
  if (myPower != -1) myPower -= m.myPower;
  return *this;
}

inline bool Monom::isOneVariable(size_t num) const {
  if (myPowers.empty()) return true;
  if ((myPowers.size() == 1) && (myPowers.begin()->first == num)) return true;
  return false;
}

//--------------------------------------------------------Polynom----------------------------------

template <typename T, typename Cmp>
inline Polynom<T, Cmp>::Polynom() {}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>::Polynom(Polynom<T, Cmp> const& p) {
  myNoms = p.myNoms;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>::Polynom(Monom const& m) {
  myNoms[m]=1;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator=(const Polynom<T, Cmp>& p) {
  myNoms = p.myNoms;
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator=(const Monom& m) {
  myNoms.clear();
  myNoms[m]=1;
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator+=(const T& i) {
  addNom(Monom(), i);
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator-=(const T& i) {
  addNom(Monom(), -i);
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator+=(const Monom& m) {
  typename std::map<Monom,T, Cmp>::iterator it = myNoms.find(m);
  if (it == myNoms.end()) {
    myNoms[m] = 1;
  } else {
  T value = it->second + 1;
    if (value != 0) {
      it->second = value;
    } else {
      myNoms.erase(it);
    }
  }
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator-=(const Monom& m) {
  typename std::map<Monom,T, Cmp>::iterator it = myNoms.find(m);
  if (it == myNoms.end()) {
    myNoms[m] = -1;
  } else {
  T value = it->second - 1;
    if (value != 0) {
      it->second = value;
    } else {
      myNoms.erase(it);
    }
  }
  return *this;
}

template <typename T, typename Cmp>
inline void Polynom<T, Cmp>::addNom(Monom m, const T& k) {
  typename std::map<Monom,T, Cmp>::iterator it = myNoms.find(m);
  if ((it == myNoms.end()) && (k != 0)) {
    myNoms[m] = k;
  } else {
  T value = it->second + k;
    if (value != 0) {
      it->second = value;
    } else {
      myNoms.erase(it);
    }
  }
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator-() const {
  Polynom temp = *this;
  temp*=-1;
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator+(const Polynom<T, Cmp> &p) const {
  Polynom temp = *this;
  temp+=p;
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator-(const Polynom<T, Cmp> &p) const {
  Polynom temp = *this;
  temp-=p;
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator+(const Monom &m) const {
  Polynom<T, Cmp> temp = *this;
  temp+=m;
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator-(const Monom &m) const {
  Polynom<T, Cmp> temp = *this;
  temp-=m;
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator*(const Monom &m) const {
  Polynom<T, Cmp> temp = *this;
  temp*=m;
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator*(const T& i) const {
  Polynom<T, Cmp> temp = *this;
  temp*=i;
  return temp;
}

template <typename T, typename Cmp>
inline bool Polynom<T, Cmp>::isNull() const {
  return myNoms.empty();
}

template <typename T, typename Cmp>
inline bool Polynom<T, Cmp>::isNumber() const {
  return (LM().getPower() == 0);
}

template <typename T, typename Cmp>
inline T* Polynom<T, Cmp>::getValue(T& value, T& denominator, const Polynombasis<T, Cmp>& ps) {
  Polynom<T, Cmp> temp;
  Polynombasis<T, Cmp> ps1 = ps.makeBasis_h();
  temp = getRemainder(ps1, denominator);
  if (temp.isNull()) {
    value = 0;
    denominator = 1;
    return &value;
  }
  if (temp.isNumber()) {
    value = temp.LC();
    T temp1 = gcd(value, denominator);
    value/=temp1;
    denominator/=temp1;
    return &value;
  } else {
    return 0;
  }
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator*=(const Polynom<T, Cmp>& p) {
  std::map<Monom,T, Cmp> temp, temp1;
  typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
  for (typename std::map<Monom,T, Cmp>::iterator it = myNoms.begin(); it != iend; ++it) {
    temp1 = p.myNoms;
    typename std::map<Monom,T, Cmp>::const_iterator tend = temp1.end();
    for (typename std::map<Monom,T, Cmp>::iterator that = temp1.begin(); that != tend; ++that) {
      Monom* m1 = (Monom*) &that->first;
      (*m1) *= it->first;
      that->second *= it->second;
    }
    MapAdd(temp, temp1);
  }
  myNoms=temp;
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::operator*(const Polynom<T, Cmp> &p) const {
  Polynom<T, Cmp> temp;
  std::map<Monom,T, Cmp> temp1;
  typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
  for (typename std::map<Monom,T, Cmp>::const_iterator it = myNoms.begin(); it != iend; ++it) {
    temp1 = p.myNoms;
    typename std::map<Monom,T, Cmp>::const_iterator tend = temp1.end();
    for (typename std::map<Monom,T, Cmp>::iterator that = temp1.begin(); that != tend; ++that) {
      Monom* m1 = (Monom*) &that->first;
      (*m1) *= it->first;
      that->second *= it->second;
    }
    MapAdd(temp.myNoms, temp1);
  }
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator*=(const Monom& m) {
  //typename std::map<Monom,T, Cmp> temp;
  if (m.getPower() != 0) {
    typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
    for (typename std::map<Monom,T, Cmp>::iterator it = myNoms.begin(); it != iend; ++it) {
      Monom* m1 = (Monom*) &it->first;
      (*m1) *= m;
     // temp[m1]=it->second;
    }
 // myNoms=temp;
  }
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator*=(const T& i) {
  if (i == 0) {
    myNoms.clear();
  } else {
    typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
    for (typename std::map<Monom,T, Cmp>::iterator it = myNoms.begin(); it != iend; ++it) {
      it->second *= i;
    }
  }
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator+=(const Polynom<T, Cmp>& p) {
  MapAdd(myNoms, p.myNoms);
  return *this;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp>& Polynom<T, Cmp>::operator-=(const Polynom<T, Cmp>& p) {
  MapSub(myNoms, p.myNoms);
  return *this;
}

template <typename T, typename Cmp>
inline bool Polynom<T, Cmp>::operator == (const Polynom<T, Cmp> &p) const {
  typename std::map<Monom,T, Cmp>::const_iterator it;
  typename std::map<Monom,T, Cmp>::const_iterator that = p.myNoms.begin();
  typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
  typename std::map<Monom,T, Cmp>::const_iterator tend = p.myNoms.end();
  for (it = myNoms.begin(); (it != iend) && (that != tend); ++it) {
    if (!(it->first == that->first)) return false;
    if ((it->second) != that->second) return false;
    ++that;
  }
  if (it != iend) return false;
  if (that != tend) return false;
  return true;
}

template <typename T, typename Cmp>
inline Monom Polynom<T, Cmp>::LM() const {
  Monom temp;
  if (myNoms.empty()) {
  } else {
    temp = myNoms.rbegin()->first;
  }
  return temp;
}

template <typename T, typename Cmp>
inline T Polynom<T, Cmp>::LC() const {
  T temp;
  if (myNoms.empty()) {
    temp = 1;
  } else {
    temp = myNoms.rbegin()->second;
  }
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::getRemainder(Polynombasis<T, Cmp>& ps) const {

  Polynom<T, Cmp> p = *this;
  Polynom<T, Cmp> r;
  while (!(p.isNull())) {
    size_t i = 0;
    bool isdivision = false;
    while ((i < ps.getSize()) && !isdivision) {
      if (p.LM().divisible(ps[i].LM())) {
        if ((p.LC() % ps[i].LC()) != 0) {
          r *= (ps[i].LC()/gcd(ps[i].LC(), p.LC()));
          p *= (ps[i].LC()/gcd(ps[i].LC(), p.LC()));
        }
        Polynom<T, Cmp> temp = ps[i];
        temp *= (p.LM() / ps[i].LM());
        temp *= (p.LC() / ps[i].LC());
        p -= temp;
        isdivision = true;
      } else {

        ++i;
      }
    }
    if (!isdivision) {
      Monom m = p.LM();
      T c = p.LC();
      r.addNom(m, c);
      p.addNom(m, -c);
    }
  }
  return r;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> Polynom<T, Cmp>::getRemainder(Polynombasis<T, Cmp>& ps, T& denominator) const {

  Polynom<T, Cmp> p = *this;
  Polynom<T, Cmp> r;
  denominator = 1;
  while (!(p.isNull())) {
    size_t i = 0;
    bool isdivision = false;
    while ((i < ps.getSize()) && !isdivision) {
      if (p.LM().divisible(ps[i].LM())) {
        if ((p.LC() % ps[i].LC()) != 0) {
          r *= (ps[i].LC()/gcd(ps[i].LC(), p.LC()));
          denominator *= (ps[i].LC()/gcd(ps[i].LC(), p.LC()));
          p *= (ps[i].LC()/gcd(ps[i].LC(), p.LC()));
        }
        Polynom<T, Cmp> temp = ps[i];
        temp *= (p.LM() / ps[i].LM());
        temp *= (p.LC() / ps[i].LC());
        p -= temp;
        isdivision = true;
      } else {

        ++i;
      }
    }
    if (!isdivision) {
      Monom m = p.LM();
      T c = p.LC();
      r.addNom(m, c);
      p.addNom(m, -c);
    }
  }
  return r;
}

template <typename T, typename Cmp>
inline void Polynom<T, Cmp>::reduce() {
  if (myNoms.empty()) return;
  T k = myNoms.rbegin()->second;
  typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
  for (typename std::map<Monom,T, Cmp>::const_iterator it = myNoms.begin(); it != iend; ++it) {
    k = gcd(k, it->second);
  }
  if (myNoms.rbegin()->second < 0) k *= -1;
  for (typename std::map<Monom,T, Cmp>::iterator it = myNoms.begin(); it != iend; ++it) {
    it->second /= k;
  }
}

template <typename T, typename Cmp>
inline bool Polynom<T, Cmp>::isOneVariable(size_t num) const {
  typename std::map<Monom,T, Cmp>::const_iterator iend = myNoms.end();
  for (typename std::map<Monom,T, Cmp>::const_iterator it = myNoms.begin(); it != iend; ++it) {
    if (!it->first.isOneVariable(num)) return false;
  }
  return true;
}

//------------------------------------------Polynombasis------------------------------------

template <typename T, typename Cmp>
inline Polynombasis<T, Cmp>::Polynombasis() {}

template <typename T, typename Cmp>
inline Polynombasis<T, Cmp>::Polynombasis(Polynombasis const& p) {
  myPolynoms = p.myPolynoms;
}

template <typename T, typename Cmp>
inline void Polynombasis<T, Cmp>::addPolynom(const Polynom<T, Cmp> &p) {
  Polynom<T, Cmp> p1 = p;
  p1.reduce();
  if (!p1.isNull() && !contains(p1)) {
    myPolynoms.push_back(p1);
  }
}

template <typename T, typename Cmp>
inline Polynombasis<T, Cmp>& Polynombasis<T, Cmp>::operator=(const Polynombasis<T, Cmp>& p) {
  myPolynoms = p.myPolynoms;
  return *this;
}

template <typename T, typename Cmp>
inline const Polynom<T, Cmp>& Polynombasis<T, Cmp>::operator[](size_t i) const {
  return myPolynoms[i];
}

template <typename T, typename Cmp>
inline size_t Polynombasis<T, Cmp>::getSize() const {
  return myPolynoms.size();
}

template <typename T, typename Cmp>
struct pComp {
  bool operator() (const Polynom<T, Cmp>& i, const Polynom<T, Cmp>& j) {
    return Cmp()(i.LM(), j.LM());
  }
};

template <typename T, typename Cmp>
inline void Polynombasis<T, Cmp>::sort() {
  pComp<T, Cmp> Comp;

  std::sort(myPolynoms.begin(), myPolynoms.end(), Comp);

}

template <typename T, typename Cmp>
inline bool Polynombasis<T, Cmp>::contains(const Polynom<T, Cmp> &p) const {
  Polynom<T, Cmp> p1 = p;
  p1.reduce();
  typename std::vector<Polynom<T, Cmp> >::const_iterator iend = myPolynoms.end();
  for (typename std::vector<Polynom<T, Cmp> >::const_iterator it = myPolynoms.begin(); it != iend; ++it) {
    if (p1 == *it) return true;
  }
  return false;
}

template <typename T, typename Cmp>
inline bool criterion(size_t i, size_t j, const Pairs &B, const Polynombasis<T, Cmp> &p, const Monom& m) {
  size_t g = p.getSize();
  for (size_t l = 0; l != i; ++l) {
    if ((!B.contains(l, i))  && (!B.contains(l, j)) &&  m.divisible(p[l].LM())) return true;
  }
  for (size_t l = i + 1; l != j; ++l) {
    if ((!B.contains(i, l))  && (!B.contains(l, j)) &&  m.divisible(p[l].LM())) return true;
  }
  for (size_t l = j + 1; l != g; ++l) {
    if ((!B.contains(i, l))  && (!B.contains(j, l)) &&  m.divisible(p[l].LM())) return true;
  }
  return false;
}

template <typename T, typename Cmp>
inline Polynombasis<T, Cmp> Polynombasis<T, Cmp>::makeBasis() const{
  Pairs B;
  for (size_t i = 0; i != (myPolynoms.size() - 1); ++i) {
    for (size_t j = i+1; j != myPolynoms.size(); ++j) {
      B.add(i, j, LCM(myPolynoms[i].LM(), myPolynoms[j].LM()));
    }
  }
  Polynombasis<T, Cmp> g = *this;
  size_t t = myPolynoms.size();
  while (!B.empty()) {
    size_t i = B.getfirstI();
    size_t j = B.getfirstJ();
    Monom m = B.getfirstM();
    B.removefirst();
    if (!(m == g[i].LM() * g[j].LM()) && !criterion(i, j, B, g, m)) {
      Polynom<T, Cmp> s = S(g[i], g[j]);
      s = s.getRemainder(g);
      if (!s.isNull()) {
        ++t;
        g.addPolynom(s);
        for (size_t ii = 0; ii != t-1; ++ii) {
          B.add(ii, t-1, LCM(g[ii].LM(), g[t-1].LM()));
        }
      }
    }
/*
    bool f = true;
    while (f) {
      f = false;
      for (size_t i1 = 0; i1 != g.myPolynoms.size(); ++i1) {
        Polynom<T, Cmp> gg = g[i1];
        Polynombasis<T, Cmp> g1;
        for (size_t j1 = 0; j1 != i1; ++j1) {
          g1.addPolynom(g[j1]);
        }
        for (size_t j1 = i1 + 1; j1 != g.myPolynoms.size(); ++j1) {
          g1.addPolynom(g[j1]);
        }
        Polynom<T, Cmp> gg1;
        gg1 = gg.getRemainder(g1);
        if (!(gg == gg1)) f = true;
        g1.myPolynoms.clear();
        for (size_t j1 = 0; j1 != i1; ++j1) {
          g1.addPolynom(g[j1]);
        }
        g1.addPolynom(gg1);
        for (size_t j1 = i1 + 1; j1 != g.myPolynoms.size(); ++j1) {
          g1.addPolynom(g[j1]);
        }
        if (gg1.isNull()) {
          B.removenum(i1--);
          --t;
        }
        g = g1;
      }
    }
*/
  }
  g.reduce();
  g.sort();
  return g;
}

template <typename T, typename Cmp>
inline bool criterion(size_t i, size_t j, const Pairs_h &B, const Polynombasis<T, Cmp> &p, const Monom& m) {
  size_t g = p.getSize();
  for (size_t l = 0; l != i; ++l) {
    if ((!B.contains(l, i))  && (!B.contains(l, j)) &&  m.divisible(p[l].LM())) return true;
  }
  for (size_t l = i + 1; l != j; ++l) {
    if ((!B.contains(i, l))  && (!B.contains(l, j)) &&  m.divisible(p[l].LM())) return true;
  }
  for (size_t l = j + 1; l != g; ++l) {
    if ((!B.contains(i, l))  && (!B.contains(j, l)) &&  m.divisible(p[l].LM())) return true;
  }
  return false;
}

template <typename T, typename Cmp>
inline Polynombasis<T, Cmp> Polynombasis<T, Cmp>::makeBasis_h() const{
  Pairs_h B;
  for (size_t i = 0; i != (myPolynoms.size() - 1); ++i) {
    for (size_t j = i+1; j != myPolynoms.size(); ++j) {
      B.add(i, j, LCM(myPolynoms[i].LM(), myPolynoms[j].LM()));
    }
  }
  Polynombasis<T, Cmp> g = *this;
  size_t t = myPolynoms.size();
  while (!B.empty()) {
    size_t i = B.getfirstI();
    size_t j = B.getfirstJ();
    Monom m = B.getfirstM();
    B.removefirst();
    if (!(m == g[i].LM() * g[j].LM()) && !criterion(i, j, B, g, m)) {
      Polynom<T, Cmp> s = S(g[i], g[j]);
      s = s.getRemainder(g);
      if (!s.isNull()) {
        ++t;
        g.addPolynom(s);
        for (size_t ii = 0; ii != t-1; ++ii) {
          B.add(ii, t-1, LCM(g[ii].LM(), g[t-1].LM()));
        }
      }
    }
  }
  g.reduce();
  g.sort();
  return g;
}

template <typename T, typename Cmp>
inline void Polynombasis<T, Cmp>::reduce() {
  bool f = true;
  while (f) {
    f = false;
    size_t s = myPolynoms.size();
    for (size_t i = 0; i != s; ++i) {
      Polynom<T, Cmp> gg = myPolynoms[i];
      Polynombasis<T, Cmp> g1;
      for (size_t j = 0; j != i; ++j) {
        g1.addPolynom(myPolynoms[j]);
      }
      for (size_t j = i + 1; j != s; ++j) {
        g1.addPolynom(myPolynoms[j]);
      }
      Polynom<T, Cmp> gg1;
      gg1 = gg.getRemainder(g1);
      if (!(gg == gg1)) {
        f = true;
        if (gg1.isNull()) {
          g1.myPolynoms.clear();
          for (size_t j = 0; j != i; ++j) {
            g1.addPolynom(myPolynoms[j]);
          }
          for (size_t j = i + 1; j != s; ++j) {
            g1.addPolynom(myPolynoms[j]);
          }
          --i;
          --s;
         myPolynoms = g1.myPolynoms;
        } else {
          gg1.reduce();
          myPolynoms[i] = gg1;
        }
      }
    }
  }
}

template <typename T, typename Cmp>
inline bool Polynombasis<T, Cmp>::isNumber() {
  return myPolynoms[0].isNumber();
}

//------------------------------------------------Basispair

inline bool Basispair::operator<(const Basispair& p) const {
/*  if (I<p.I) return true;
  if (I>p.I) return false;
  if (J<p.J) return true;
  if (J>p.J) return false;
  return false;*/
  return (IJ < p.IJ);
}

//------------------------------------------------Basismonom

inline bool Basismonom::operator<(const Basismonom& m) const {
  return grevlexC()(M, m.M);
}

//------------------------------------------------Pairs

inline void Pairs::add(size_t i, size_t j, const Monom& m) {
  Basispair tempP;
  Basismonom tempM;
  tempP.I = i;
  tempP.J = j;
  tempP.IJ = 65536 * i + j;
  tempM.M = m;
  std::pair<std::set<Basispair>::iterator,bool> p = myPairs.insert(tempP);
  if (p.second == 0) {
    printf("AAAAA!!!!!!!!!11\n");
    return;
  }
  tempM.toPair = p.first;
  std::multiset<Basismonom>::iterator it = myMonoms.insert(tempM);
  Basispair* b = (Basispair*)&*(p.first);
  b->toMonom = it;
  ++mySize;
}

inline size_t Pairs::getfirstI() const {
  return (*myMonoms.begin()).toPair->I;
}

inline size_t Pairs::getfirstJ() const {
  return (*myMonoms.begin()).toPair->J;
}

inline Monom Pairs::getfirstM() const {
  return (*myMonoms.begin()).M;
}

inline void Pairs::removefirst() {
  if (!empty()) {
    myPairs.erase(myMonoms.begin()->toPair);
    myMonoms.erase(myMonoms.begin());
    --mySize;
  }
}

inline bool Pairs::empty() const {
 // return myData.empty();
  return (mySize == 0);
}

inline bool Pairs::contains(size_t i, size_t j) const {
  Basispair temp;
/*  temp.I = i;
  temp.J = j;*/
  temp.IJ = 65536 * i + j;
  std::set<Basispair>::iterator it = myPairs.find(temp);
  if (it == myPairs.end()) return false;
  return true;
}



/*inline void Pairs::removenum(size_t num) {
  for (std::multiset<Basispair, pairC>::iterator it = myData.begin(); it != myData.end();) {
    if ((it->getI() == num) || (it->getJ() == num)) {
      myData.erase(it++);
    } else {
       if (it->myI > num) {
         --(*((size_t*)&(it->myI)));
       }
       if (it->myJ > num) {
         --(*((size_t*)&(it->myJ)));
       }
       ++it;
    }
  }
}*/

//------------------------------------------------Basispair_h

inline bool Basispair_h::operator==(const Basispair_h& p) const {
  return P == p.P;
}

//------------------------------------------------Basismonom_h

inline bool Basismonom_h::operator<(const Basismonom_h& m) const {
  return grevlexC()(M, m.M);
}

//------------------------------------------------Pairs_h

inline void Pairs_h::add(size_t i, size_t j, const Monom& m) {
  Basispair_h tempP;
  Basismonom_h tempM;
  tempP.P = std::make_pair(i, j);
  tempM.M = m;
  std::pair<__gnu_cxx::hash_set<Basispair_h,Pairhash>::iterator,bool> p = myPairs.insert(tempP);
  if (p.second == 0) {
    printf("AAAAA!!!!!!!!!11\n");
    return;
  }
  tempM.toPair = p.first;
  std::multiset<Basismonom_h>::iterator it = myMonoms.insert(tempM);
  Basispair_h* b = (Basispair_h*)&*(p.first);
  b->toMonom = it;
  ++mySize;
}

inline size_t Pairs_h::getfirstI() const {
  return (*myMonoms.begin()).toPair->P.first;
}

inline size_t Pairs_h::getfirstJ() const {
  return (*myMonoms.begin()).toPair->P.second;
}

inline Monom Pairs_h::getfirstM() const {
  return (*myMonoms.begin()).M;
}

inline void Pairs_h::removefirst() {
  if (!empty()) {
    myPairs.erase(myMonoms.begin()->toPair);
    myMonoms.erase(myMonoms.begin());
    --mySize;
  }
}

inline bool Pairs_h::empty() const {
 // return myData.empty();
  return (mySize == 0);
}

inline bool Pairs_h::contains(size_t i, size_t j) const {
  Basispair_h temp;
  temp.P = std::make_pair(i, j);
  __gnu_cxx::hash_set<Basispair_h,Pairhash>::iterator it = myPairs.find(temp);
  if (it == myPairs.end()) return false;
  return true;
}

//------------------------------------<other>----------------------

inline Monom LCM(const Monom& a, const Monom& b) {
  Monom temp;
  std::map<size_t, size_t>::const_iterator it = a.myPowers.begin();
  std::map<size_t, size_t>::const_iterator that = b.myPowers.begin();
  std::map<size_t, size_t>::const_iterator iend = a.myPowers.end();
  std::map<size_t, size_t>::const_iterator tend = b.myPowers.end();
  while ((it != iend) || (that != tend)) {
    if (it == iend) {
      temp[that->first] = that->second;
      ++that;
    } else {
      if (that == tend) {
        temp[it->first] = it->second;
        ++it;
      } else {
        if (it->first == that->first) {
          if (it->second > that->second) temp[it->first] = it->second;
          else temp[it->first] = that->second;
          ++it;
          ++that;
        } else {
          if (it->first < that->first) {
            temp[it->first] = it->second;
            ++it;
          } else {
            if (it->first > that->first) {
              temp[that->first] = that->second;
              ++that;
            }
          }
        }
      }
    }
  }
  return temp;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> S(const Polynom<T, Cmp> &f, const Polynom<T, Cmp> &g) {
  Polynom<T, Cmp> temp;
  Monom m;
  m = LCM(f.LM(), g.LM());
  temp = (f*(m/f.LM()))*g.LC() - (g*(m/g.LM()))*f.LC();
  return temp;
}

inline bool lexC::operator()(const Monom& m1, const Monom& m2) {
  std::map<size_t, size_t>::const_iterator it;
  std::map<size_t, size_t>::const_iterator that = m2.myPowers.begin();
  std::map<size_t, size_t>::const_iterator iend = m1.myPowers.end();
  std::map<size_t, size_t>::const_iterator tend = m2.myPowers.end();
  for (it = m1.myPowers.begin(); (it != iend) && (that != tend); ++it) {
    if (it->first < that->first) {return false;}
    if (it->first > that->first) {return true;}
    if (it->second > that->second) {return false;}
    if (it->second < that->second) {return true;}
    ++that;
  }
  if (that != m2.myPowers.end()) return true;
  return false;
}

inline bool grlexC::operator()(const Monom& m1, const Monom& m2) {
  if (m1.getPower() < m2.getPower()) return true;
  if (m1.getPower() > m2.getPower()) return false;
  std::map<size_t, size_t>::const_iterator it;
  std::map<size_t, size_t>::const_iterator that = m2.myPowers.begin();
  std::map<size_t, size_t>::const_iterator iend = m1.myPowers.end();
  std::map<size_t, size_t>::const_iterator tend = m2.myPowers.end();
  for (it = m1.myPowers.begin(); (it != iend) && (that != tend); ++it) {
    if (it->first < that->first) {return false;}
    if (it->first > that->first) {return true;}
    if (it->second > that->second) {return false;}
    if (it->second < that->second) {return true;}
    ++that;
  }
  return false;
}

inline bool grevlexC::operator()(const Monom& m1, const Monom& m2) {
  if (m1.getPower() < m2.getPower()) return true;
  if (m1.getPower() > m2.getPower()) return false;
  std::map<size_t, size_t>::const_reverse_iterator it;
  std::map<size_t, size_t>::const_reverse_iterator that = m2.myPowers.rbegin();
  std::map<size_t, size_t>::const_reverse_iterator iend = m1.myPowers.rend();
  std::map<size_t, size_t>::const_reverse_iterator tend = m2.myPowers.rend();
  for (it = m1.myPowers.rbegin(); (it != iend) && (that != tend); ++it) {
    if (it->first < that->first) {return false;}
    if (it->first > that->first) {return true;}
    if (it->second > that->second) {return true;}
    if (it->second < that->second) {return false;}
    ++that;
  }
  return false;
}

//-------------------------------VarDump--------------

inline size_t VarDump::addVar(const std::string& s) {
  std::vector<std::string>::iterator it = std::find(myData.begin(), myData.end(), s);
  if (it == myData.end()) {
    myData.push_back(s);
    it = myData.end() - 1;
  }
  return (it - myData.begin());
}

inline size_t VarDump::getNum(const std::string& s) const {
  std::vector<std::string>::const_iterator it = std::find(myData.begin(), myData.end(), s);
  if (it == myData.end()) {
    return -1;
  }
  return (it - myData.begin());
}

inline const std::string& VarDump::getVar(size_t n) const {
  static std::string Error = "OMG!!! This variable has no name!";
  if (n <= myData.size()) {
    return myData[n];
  }
  return Error;
}

inline std::string VarDump::Monomtostr(const Monom& m) const {
  if (m.getPower() == 0) {
    return "1";
  } else {
    std::string s;
    std::stringstream out;
    for (std::map<size_t, size_t>::const_iterator it = m.myPowers.begin(); it != m.myPowers.end(); ++it) {
      if (it->second != 0) {
        if (it->second != 1) {
          out << getVar(it->first) << '^' << it->second;
          s = out.str();
        } else {
          out << getVar(it->first);
          s = out.str();
        }
      }
      std::map<size_t, size_t>::const_iterator it1 = it;
      ++it1;
      if (it1 != m.myPowers.end()) {
          out << '*';
          s = out.str();
      }
    }
    return s;
  }
}

inline Monom VarDump::strtoMonom(const std::string& s) {
  size_t i = 0;
  Monom m;
  size_t pw;
  while (i < s.size()) {
    std::string s1;
    pw = 0;
    while ((s[i] != '*') && (s[i] != '^') && (i < s.size())) {     //чтение переменной
      s1 += s[i];
      ++i;
    }
    if ((i < s.size()) && (s[i] == '^')) {
      ++i;
      while (isdigit(s[i]) && (i < s.size())) {     //чтение степени
        pw = pw * 10 + s[i] -'0';
        ++i;
      }
      if ((i < s.size()) && (s[i] == '*')) {++i;}
    } else {
      if ((i < s.size()) && (s[i] == '*')) {
        pw = 1;
        ++i;
      } else {
        if (i == s.size()) {
          pw = 1;
        }
      } 
    }
    m[addVar(s1)] = pw;
  }
  return m;
}

template <typename T, typename Cmp>
inline std::string VarDump::Polynomtostr(const Polynom<T, Cmp>& p) const {
  if (p.myNoms.empty()) {
    return "0";
  } else {
    std::string s;
    std::stringstream out;
    for (typename std::map<Monom,T, Cmp>::const_reverse_iterator it = p.myNoms.rbegin(); it != p.myNoms.rend(); ++it) {
      if (it->first.getPower() == 0) {
        out << it->second;
        s = out.str();
        return s;
      }
      if ((it->second != 1) && (it->second != -1)) {out << it->second;}
      if (it->second == -1) {out << '-';}
      out << Monomtostr(it->first);
      typename std::map<Monom,T, Cmp>::const_reverse_iterator it1 = it;
      ++it1;
      if (it1 != p.myNoms.rend()) {
        out << ' ';
        if (it1->second >= 0) {
          out << '+';
        }
      }
    }
    s = out.str();
    return s;
  }
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> VarDump::strtoPolynom(const std::string& s) {
  Polynom<T, Cmp> p;
  size_t i = 0;
  bool plus;
  if (s[0] == '-') {
    plus = false;
    i = 1;
  } else {
    plus = true;
  }
  T k;
  while (i < s.size()) {
    Monom m;
    k = 0;
    if (!isdigit(s[i])) {
      k = 1;
    } else {
      while (isdigit(s[i]) && (i < s.size())) {  //чтение коэффициента
        k = k * 10 + s[i] - '0';
        ++i;
      }
    }
    std::string s1;
    while ((s[i] != ' ') && (i < s.size())) {
      s1 += s[i];
      ++i;
    }
    m = strtoMonom(s1);
    if (plus) {
      if (k != 0) p.addNom(m, k);
    } else {
      if (k != 0) p.addNom(m, -k);
    }
    if (i < s.size()) {++i;}
    if (s[i] == '+') {
      plus = true;
      ++i;
    } else {
      plus = false;
      ++i;
    };
  }
  return p;
}

template <typename T, typename Cmp>
inline Polynom<T, Cmp> VarDump::freadPolynom(FILE* f) {
  char* c = new char[8192];
  std::string s;
  fgets(c, 8192, f);
  s=c;
  delete[] c;
  s.erase(s.length() - 1);
  return strtoPolynom<T, Cmp>(s);
}

template <typename T, typename Cmp>
inline void VarDump::printPolynom(const Polynom<T, Cmp>& p) {
  printf("%s", Polynomtostr<T, Cmp>(p).c_str());
}

template <typename T, typename Cmp>
inline void VarDump::fprintPolynom(FILE* f, const Polynom<T, Cmp>& p) {
  fprintf(f, "%s", Polynomtostr<T, Cmp>(p).c_str());
}

template <typename T, typename Cmp>
inline void VarDump::fprintBasis(FILE* f, const Polynombasis<T, Cmp>& p) {
  for (typename std::vector<Polynom<T, Cmp> >::const_iterator it = p.myPolynoms.begin(); it != p.myPolynoms.end(); ++it) {
    fprintPolynom<T, Cmp>(f, *it);
    fprintf(f, "\n");
  }
}

template <typename T, typename Cmp>
inline void VarDump::printBasis(const Polynombasis<T, Cmp>& p) {
  for (typename std::vector<Polynom<T, Cmp> >::const_iterator it = p.myPolynoms.begin(); it != p.myPolynoms.end(); ++it) {
    printPolynom<T, Cmp>(*it);
    printf("\n");
  }
}

//-----------------

template <typename T>
inline Polynom<T, lexC> getPolynom(const Polynombasis<T, lexC>& ps, const Polynom<T, lexC>& p, size_t num) {
  Polynombasis<T, lexC> ps1 = ps;
  ps1.addPolynom(p);
  ps1 = ps1.makeBasis_h();
  if (ps1[0].isOneVariable(num))return ps1[0];
  else return Polynom<T, lexC> ();
}

#endif
