#include "converter.h"
#include <ctype.h>
#include <stdio.h>
#include <iostream>
#include <math.h>
#include "../Polynom.impl.h"

std::string itoa(int value) {
  std::string buf;
  enum { kMaxDigits = 35 };
  buf.reserve( kMaxDigits );
  int quotient = value;
  do {
    buf += "0123456789abcdef"[std::abs( quotient % 10) ];
    quotient /= 10;
  } while ( quotient );
  if ( value < 0) buf += '-';
  std::reverse( buf.begin(), buf.end() );
  return buf;
}


Recognizer::Recognizer() {
  Converter* c = new equalConverter;
  myData["pp=pp"] = c;
  c = new equalNumConverter;
  myData["pp=n"] = c;
  c = new perpConverter;
  myData["pp-|pp"] = c;
  c = new perpConverter;
  myData["pp|-pp"] = c;
  c = new parConverter;
  myData["pp||pp"] = c;
  c = new lineConverter;
  myData["-ppp-"] = c;
  c = new divConverter;
  myData["p=pp/n"] = c;
  c = new angleConverter;
  myData["<ppp=<ppp"] = c;

  c = new equalProbConverter;
  myData["pp=pp?"] = c;
  c = new equalNumProbConverter;
  myData["pp=?"] = c;
  c = new perpProbConverter;
  myData["pp-|pp?"] = c;
  c = new perpProbConverter;
  myData["pp|-pp?"] = c;
  c = new parProbConverter;
  myData["pp||pp?"] = c;
  c = new lineProbConverter;
  myData["-ppp-?"] = c;
  c = new angleProbConverter;
  myData["<ppp=<ppp?"] = c;
}

Recognizer::~Recognizer() {
  for (std::map<std::string, Converter*>::iterator it = myData.begin(); it != myData.end(); ++it) {
    delete it->second;
  }
}



void Recognizer::recognize(const std::string& s) {
  std::string s1, ss;
  std::vector<std::string> v;
  size_t i = 0;
  while (i < s.size()) {

    if (isupper(s[i])) { //point
      ss.clear();
      s1 += 'p';
      ss += s[i];
      ++i;
      while ((i < s.size()) && isalnum(s[i]) && !isupper(s[i])) {
        ss += s[i];
        ++i;
      }
      v.push_back(ss);
    } else {

      if (ispunct(s[i])) {
        s1 += s[i];
        ++i;
      } else {

       if (islower(s[i])) { //variable
          ss.clear();
          s1 += 'v';
          ss += s[i];
          ++i;
          while ((i < s.size()) && isalnum(s[i])) {
            ss += s[i];
            ++i;
          }
          v.push_back(ss);
        } else {

          if (isdigit(s[i])) { //number
            ss.clear();
            s1 += 'n';
            ss += s[i];
            ++i;
            while ((i < s.size()) && isdigit(s[i])) {
              ss += s[i];
              ++i;
            }
            v.push_back(ss);
          } else {
            ++i;
          }
        }
      }
    }
  }
  //printf("%s\n", s1.c_str());
  std::map<std::string, Converter*>::const_iterator it = myData.find(s1);
  if (it == myData.end()) {
    printf("Unable to recognize string '%s'\n", s.c_str());
  } else {
    it->second->convert(v, vd, cond, prob, s);
  }
}

void Recognizer::compute() {
  output.t.clear();
  output.p.clear();
  for (size_t i = 0; i < prob.p.size(); ++i) {
    output.p.push_back(prob.p[i]);
    output.c.push_back(prob.c[i]);
    if (prob.t[i] == "square") {
      GMPint val, den;
      GMPint* j = prob.p[i].getValue(val, den, cond);
      if (j == 0) {
        output.t.push_back("unknown");
      } else {
        if ((val < 0) || (den <= 0)) {
          output.t.push_back("unknown");
        } else {
          int ival = val.getlongint();
          int iden = den.getlongint();
          double dval = sqrt(ival);
          double dden = sqrt(iden);
          int idval = (int)dval;
          int idden = (int)dden;
          if (((idval * idval) == ival) && ((idden * idden) == iden)) {
            std::stringstream ss;
            ss << idval;
            if (idden != 1) {
              ss << "/" << idden;
            }
            output.t.push_back(ss.str());
          } else {
            std::stringstream ss;
            ss << "√(" << ival;
            if (iden != 1) {
              ss << "/" << iden << ")";
            } else ss << ")";
            output.t.push_back(ss.str());
          }
        }
      }
    }
    if (prob.t[i] == "y/n") {
      Polynombasis<GMPint, grevlexC> P = cond;
      Polynom<GMPint, grevlexC> temp = prob.p[i];
      temp *= vd.strtoPolynom<GMPint, grevlexC>("unusedY");
      temp = -temp;
      temp += 1;
      P.addPolynom(temp);
      P = P.makeBasis2();
      P = P.reduce();
      if (P.isNumber()) {
        output.t.push_back("true");
      } else {
        output.t.push_back("false");
      }
    }
  }
}

void Recognizer::read(FILE* f) {
  char* c = new char[8192];
  std::string s;
  bool notend = true;
  notend = fgets(c, 8192, f);
  while (notend) {
    s = c;
    s.erase(s.length() - 1);
    recognize(s);
    notend = fgets(c, 8192, f);
  }
  delete[] c;
}

void Recognizer::write(FILE* f) {
  for (size_t i = 0; i < output.p.size(); ++i) {
    fprintf(f, "%s %s\n", output.c[i].c_str(), output.t[i].c_str());
  }
}

void equalConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr;
  tempstr = 'x' + v[0] + " -" + 'x' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'y' + v[0] + " -" + 'y' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'x' + v[2] + " -" + 'x' + v[3];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'y' + v[2] + " -" + 'y' + v[3];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  cond.addPolynom(temp);
  //vd.printPolynom<GMPint, grevlexC>(temp);
 // printf("\n");
}

void equalNumConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr;
  int n;
  tempstr = 'x' + v[0] + " -" + 'x' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'y' + v[0] + " -" + 'y' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  n = atoi(v[2].c_str());
  n *= n;
  temp -= n;
  cond.addPolynom(temp);
 // vd.printPolynom<GMPint, grevlexC>(temp);
//  printf("\n");
}

void perpConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr1, tempstr2;
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[1];
  tempstr2 = 'x' + v[2] + " -" + 'x' + v[3];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'y' + v[0] + " -" + 'y' + v[1];
  tempstr2 = 'y' + v[2] + " -" + 'y' + v[3];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  cond.addPolynom(temp);
//  vd.printPolynom<GMPint, grevlexC>(temp);
//  printf("\n");
}

void parConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr1, tempstr2;
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[1];
  tempstr2 = 'y' + v[2] + " -" + 'y' + v[3];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'x' + v[2] + " -" + 'x' + v[3];
  tempstr2 = 'y' + v[0] + " -" + 'y' + v[1];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  cond.addPolynom(temp);
//  vd.printPolynom<GMPint, grevlexC>(temp);
//  printf("\n");
}

void lineConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr1, tempstr2;
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[1];
  tempstr2 = 'y' + v[0] + " -" + 'y' + v[2];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[2];
  tempstr2 = 'y' + v[0] + " -" + 'y' + v[1];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  cond.addPolynom(temp);
  //vd.printPolynom<GMPint, grevlexC>(temp);
 // printf("\n");
}

void divConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr;
  int n;
  n = atoi(v[3].c_str());
  tempstr = itoa(n-1) + 'x' + v[1] + " +" + 'x' + v[2] + " -" + itoa(n) + 'x' + v[0];
  temp = (vd.strtoPolynom<GMPint, grevlexC>(tempstr));
  cond.addPolynom(temp);
  tempstr = itoa(n-1) + 'y' + v[1] + " +" + 'y' + v[2] + " -" + itoa(n) + 'y' + v[0];
  temp = (vd.strtoPolynom<GMPint, grevlexC>(tempstr));
  cond.addPolynom(temp);
 // vd.printPolynom<GMPint, grevlexC>(temp);
  //printf("\n");
}

void angleConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp,temp1,temp11;
  std::string tempstr1,tempstr2;
  tempstr1 = 'x' + v[1] + " -" + 'x' + v[0];
  tempstr2 = 'x' + v[1] + " -" + 'x' + v[2];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'y' + v[1] + " -" + 'y' + v[0];
  tempstr2 = 'y' + v[1] + " -" + 'y' + v[2];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  temp1 = temp11;
  tempstr1 = 'x' + v[4] + " -" + 'x' + v[3];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[4] + " -" + 'y' + v[3];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  tempstr1 = 'x' + v[4] + " -" + 'x' + v[5];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[4] + " -" + 'y' + v[5];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  temp = temp1;
  tempstr1 = 'x' + v[4] + " -" + 'x' + v[3];
  tempstr2 = 'x' + v[4] + " -" + 'x' + v[5];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'y' + v[4] + " -" + 'y' + v[3];
  tempstr2 = 'y' + v[4] + " -" + 'y' + v[5];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  temp1 = temp11;
  tempstr1 = 'x' + v[1] + " -" + 'x' + v[0];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[1] + " -" + 'y' + v[0];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  tempstr1 = 'x' + v[1] + " -" + 'x' + v[2];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[1] + " -" + 'y' + v[2];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  temp -= temp1;
  cond.addPolynom(temp);
 // vd.printPolynom<GMPint, grevlexC>(temp);
//  printf("\n");
}

//--------------------prob

void equalProbConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr;
  tempstr = 'x' + v[0] + " -" + 'x' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'y' + v[0] + " -" + 'y' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'x' + v[2] + " -" + 'x' + v[3];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'y' + v[2] + " -" + 'y' + v[3];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  prob.p.push_back(temp);
  prob.t.push_back("y/n");
  prob.c.push_back(s);
  //vd.printPolynom<GMPint, grevlexC>(temp);
  //printf("\n");
}

void equalNumProbConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr;
  tempstr = 'x' + v[0] + " -" + 'x' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  tempstr = 'y' + v[0] + " -" + 'y' + v[1];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr)));
  prob.p.push_back(temp);
  prob.t.push_back("square");
  prob.c.push_back(s);
 // vd.printPolynom<GMPint, grevlexC>(temp);
 // printf("\n");
}

void perpProbConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr1, tempstr2;
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[1];
  tempstr2 = 'x' + v[2] + " -" + 'x' + v[3];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'y' + v[0] + " -" + 'y' + v[1];
  tempstr2 = 'y' + v[2] + " -" + 'y' + v[3];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  prob.p.push_back(temp);
  prob.t.push_back("y/n");
  prob.c.push_back(s);
//  vd.printPolynom<GMPint, grevlexC>(temp);
//  printf("\n");
}

void parProbConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr1, tempstr2;
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[1];
  tempstr2 = 'y' + v[2] + " -" + 'y' + v[3];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'x' + v[2] + " -" + 'x' + v[3];
  tempstr2 = 'y' + v[0] + " -" + 'y' + v[1];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  prob.p.push_back(temp);
  prob.t.push_back("y/n");
  prob.c.push_back(s);
 // vd.printPolynom<GMPint, grevlexC>(temp);
 // printf("\n");
}

void lineProbConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp;
  std::string tempstr1, tempstr2;
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[1];
  tempstr2 = 'y' + v[0] + " -" + 'y' + v[2];
  temp += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'x' + v[0] + " -" + 'x' + v[2];
  tempstr2 = 'y' + v[0] + " -" + 'y' + v[1];
  temp -= ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  prob.p.push_back(temp);
  prob.t.push_back("y/n");
  prob.c.push_back(s);
  //vd.printPolynom<GMPint, grevlexC>(temp);
 // printf("\n");
}

void angleProbConverter::convert(const std::vector<std::string>& v, VarDump& vd, Polynombasis<GMPint, grevlexC>& cond, Problems& prob, const std::string& s) {
  Polynom<GMPint, grevlexC> temp,temp1,temp11;
  std::string tempstr1,tempstr2;
  tempstr1 = 'x' + v[1] + " -" + 'x' + v[0];
  tempstr2 = 'x' + v[1] + " -" + 'x' + v[2];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'y' + v[1] + " -" + 'y' + v[0];
  tempstr2 = 'y' + v[1] + " -" + 'y' + v[2];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  temp1 = temp11;
  tempstr1 = 'x' + v[4] + " -" + 'x' + v[3];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[4] + " -" + 'y' + v[3];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  tempstr1 = 'x' + v[4] + " -" + 'x' + v[5];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[4] + " -" + 'y' + v[5];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  temp = temp1;
  tempstr1 = 'x' + v[4] + " -" + 'x' + v[3];
  tempstr2 = 'x' + v[4] + " -" + 'x' + v[5];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  tempstr1 = 'y' + v[4] + " -" + 'y' + v[3];
  tempstr2 = 'y' + v[4] + " -" + 'y' + v[5];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr2)));
  temp1 = temp11;
  tempstr1 = 'x' + v[1] + " -" + 'x' + v[0];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[1] + " -" + 'y' + v[0];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  tempstr1 = 'x' + v[1] + " -" + 'x' + v[2];
  temp11 = ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  tempstr1 = 'y' + v[1] + " -" + 'y' + v[2];
  temp11 += ((vd.strtoPolynom<GMPint, grevlexC>(tempstr1)) * (vd.strtoPolynom<GMPint, grevlexC>(tempstr1)));
  temp1 *= temp11;
  temp -= temp1;
  prob.p.push_back(temp);
  prob.t.push_back("y/n");
  prob.c.push_back(s);
 // vd.printPolynom<GMPint, grevlexC>(temp);
 // printf("\n");
}




