#include <cmath>

#include "point.h"
#include "conditions.h"
#include "gmpint.h"
#include "Polynom.h"
#include "Polynom.impl.h"
#include "polynomtask.h"

Statement::~Statement() {
}

TriangleCondition::TriangleCondition(const Point &first, const Point &second, const Point &third): myFirst(first), mySecond(second), myThird(third) {
}

DistanceCondition::DistanceCondition(const Point &first, const Point &second, const std::string &distanceVariable): myFirst(first), mySecond(second), myDistanceVariable(distanceVariable) {
}

DistanceToSegmentCondition::DistanceToSegmentCondition(const Point &point, const Point &first, const Point &second, const std::string &distanceVariable): myPoint(point), myFirst(first), mySecond(second), myDistanceVariable(distanceVariable) {
}

PointDivideSegmentCondition::PointDivideSegmentCondition(const Point &point, const Point &left, const Point &right, const std::string &coefVariable): myPoint(point), myLeft(left), myRight(right), myCoeficientVariable(coefVariable) {
}

MiddlePointCondition::MiddlePointCondition(const Point &first, const Point &second, const Point &middle): myFirst(first), mySecond(second), myMiddle(middle) {
}

AngleCondition::AngleCondition(const Point &left, const Point &vertex, const Point &right, const std::string &angleVariable): myLeft(left), myVertex(vertex), myRight(right), myAngleVariable(angleVariable) {
}

TriangleAreaCondition::TriangleAreaCondition(const Point &first, const Point &second, const Point &third, const std::string &areaVariable) : myFirst(first), mySecond(second), myThird(third), myAreaVariable(areaVariable) {
}

EqualCondition::EqualCondition(const std::string &leftVar, const std::string &rightVar) : myLeftVariable(leftVar), myRightVariable(rightVar) {
}

TimesCondition::TimesCondition(const std::string &leftVar, const std::string &rightVar, const std::string &resultVar) : myLeftVariable(leftVar), myRightVariable(rightVar), myResultVariable(resultVar) {
}

PointInSegmentCondition::PointInSegmentCondition(const Point &P, const Point &A, const Point &B) : myP(P), myA(A), myB(B) {
}

ParallelLinesCondition::ParallelLinesCondition(const Point &A, const Point &B, const Point &C, const Point &D) : myA(A), myB(B), myC(C), myD(D) {
}

DistanceQuestion::DistanceQuestion(const Point &first, const Point &second): myFirst(first), mySecond(second) {
}

AngleQuestion::AngleQuestion(const Point &left, const Point &vertex, const Point &right): myLeft(left), myVertex(vertex), myRight(right) {
}

TriangleAreaQuestion::TriangleAreaQuestion(const Point &first, const Point &second, const Point &third): myFirst(first), mySecond(second), myThird(third) {
}

TrapeziumAreaQuestion::TrapeziumAreaQuestion(const Point &A, const Point &B, const Point &C, const Point &D) : myA(A), myB(B), myC(C), myD(D) {
}

CollinearPointsCondition::CollinearPointsCondition(const Point &first, const Point &second, const Point &third): myFirst(first), mySecond(second), myThird(third) {
}

void TriangleCondition::print(std::ostream &out) const {
	out << "Triangle" << std::endl;
}

void ParallelLinesCondition::print(std::ostream &out) const {
	out << "Parallel Lines" << std::endl;
}

void CollinearPointsCondition::print(std::ostream &out) const {
	out << "Collinear points" << std::endl;
}

void DistanceCondition::print(std::ostream &out) const {
	out << "Distance between " << myFirst.getName() << " and " << mySecond.getName() << " is " << myDistanceVariable << std::endl;
}

void DistanceToSegmentCondition::print(std::ostream &out) const {
	out << "Distance to segment" << std::endl;
}

void PointDivideSegmentCondition::print(std::ostream &out) const {
	out << "Point divide segment" << std::endl;
}

void MiddlePointCondition::print(std::ostream &out) const {
	out << "Middle point of " << myFirst.getName() << " and " << mySecond.getName() << " is " << myMiddle.getName() << std::endl;
}

void AngleCondition::print(std::ostream &out) const {
	out << "Angle based on " << myLeft.getName() << ", " << myVertex.getName() << ", " << myRight.getName() << " points is " << myAngleVariable << " degrees" << std::endl;
}

void TriangleAreaCondition::print(std::ostream &out) const {
	out << "Triangle area is " << std::endl;
}

void EqualCondition::print(std::ostream &out) const {
	out << myLeftVariable << " equals " << myRightVariable << std::endl;
}

void TimesCondition::print(std::ostream &out) const {
	out << myLeftVariable << " times " << myRightVariable << " is " << myResultVariable << std::endl;
}

void PointInSegmentCondition::print(std::ostream &out) const {
	out << "Point in segment" << std::endl;
}

void DistanceQuestion::print(std::ostream &out) const {
	out << "Distance between " << myFirst.getName() << " and " << mySecond.getName() << " is ?" << std::endl;
}

void AngleQuestion::print(std::ostream &out) const {
	out << "Angle based on " << myLeft.getName() << ", " << myVertex.getName() << ", " << myRight.getName() << " points is ?" << std::endl;
}

void TriangleAreaQuestion::print(std::ostream &out) const {
	out << "Triangle area is ?" << std::endl;
}

void TrapeziumAreaQuestion::print(std::ostream &out) const {
	out << "Trapezium area is ?" << std::endl;
}

void PointDivideSegmentCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t xp = ptask.addVariable(myPoint.getX()), yp = ptask.addVariable(myPoint.getY());
	size_t x1 = ptask.addVariable(myLeft.getX()), y1 = ptask.addVariable(myLeft.getY());
	size_t x2 = ptask.addVariable(myRight.getX()), y2 = ptask.addVariable(myRight.getY());
	size_t lambda = ptask.getVariable(myCoeficientVariable);
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[xp] = 1;				p.addNom(m, 1);		m = Monom();
	m[xp] = 1;	m[lambda] = 1;		p.addNom(m, 1);		m = Monom();
	m[x1] = 1;				p.addNom(m, -1);	m = Monom();
	m[x2] = 1;	m[lambda] = 1;		p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
	p = Polynom<GMPint, grevlexC>();
	m[yp] = 1;				p.addNom(m, 1);		m = Monom();
	m[yp] = 1;	m[lambda] = 1;		p.addNom(m, 1);		m = Monom();
	m[y1] = 1;				p.addNom(m, -1);	m = Monom();
	m[y2] = 1;	m[lambda] = 1;		p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
}

void ParallelLinesCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t x1 = ptask.addVariable(myA.getX()), y1 = ptask.addVariable(myA.getY());
	size_t x2 = ptask.addVariable(myB.getX()), y2 = ptask.addVariable(myB.getY());
	size_t x3 = ptask.addVariable(myC.getX()), y3 = ptask.addVariable(myC.getY());
	size_t x4 = ptask.addVariable(myD.getX()), y4 = ptask.addVariable(myD.getY());
	size_t k1 = ptask.getVariable(ptask.getAvailableVariableName());
	size_t k2 = ptask.getVariable(ptask.getAvailableVariableName());
	size_t b1 = ptask.getVariable(ptask.getAvailableVariableName());
	size_t b2 = ptask.getVariable(ptask.getAvailableVariableName());
	size_t t = ptask.getVariable(ptask.getAvailableVariableName());
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[y1] = 1;			p.addNom(m, 1);		m = Monom();
	m[y2] = 1;			p.addNom(m, -1);	m = Monom();
	m[x1] = 1;	m[k1] = 1;	p.addNom(m, 1);		m = Monom();
	m[x2] = 1;	m[k1] = 1;	p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);	p = Polynom<GMPint, grevlexC>();
	m[y3] = 1;			p.addNom(m, 1);		m = Monom();
	m[y4] = 1;			p.addNom(m, -1);	m = Monom();
	m[x3] = 1;	m[k2] = 1;	p.addNom(m, 1);		m = Monom();
	m[x4] = 1;	m[k2] = 1;	p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);	p = Polynom<GMPint, grevlexC>();
	m[b1] = 1;			p.addNom(m, 1);		m = Monom();
	m[y1] = 1;			p.addNom(m, -1);	m = Monom();
	m[x1] = 1;	m[k1] = 1;	p.addNom(m, 1);		m = Monom();
	ptask.addPolynomialCondition(p);	p = Polynom<GMPint, grevlexC>();
	m[b2] = 1;			p.addNom(m, 1);		m = Monom();
	m[y3] = 1;			p.addNom(m, -1);	m = Monom();
	m[x3] = 1;	m[k2] = 1;	p.addNom(m, 1);		m = Monom();
	ptask.addPolynomialCondition(p);	p = Polynom<GMPint, grevlexC>();
	m[k1] = 1;			p.addNom(m, 1);		m = Monom();
	m[k2] = 1;			p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);	p = Polynom<GMPint, grevlexC>();
	m[b1] = 1;	m[t] = 1;	p.addNom(m, 1);		m = Monom();
	m[b2] = 1;	m[t] = 1;	p.addNom(m, -1);	m = Monom();
	p.addNom(Monom(), -1);
	ptask.addPolynomialCondition(p);
}

void TriangleCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t x1 = ptask.addVariable(myFirst.getX()), y1 = ptask.addVariable(myFirst.getY());
	size_t x2 = ptask.addVariable(mySecond.getX()), y2 = ptask.addVariable(mySecond.getY());
	size_t x3 = ptask.addVariable(myThird.getX()), y3 = ptask.addVariable(myThird.getY());
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[x1] = 1;	m[y3] = 1;	p.addNom(m, 1);		m = Monom();
	m[x1] = 1;	m[y2] = 1;	p.addNom(m, -1);	m = Monom();
	m[x2] = 1;	m[y1] = 1;	p.addNom(m, 1);		m = Monom();
	m[x2] = 1;	m[y3] = 1;	p.addNom(m, -1);	m = Monom();
	m[x3] = 1;	m[y2] = 1;	p.addNom(m, 1);		m = Monom();
	m[x3] = 1;	m[y1] = 1;	p.addNom(m, -1);	m = Monom();
	size_t expr = ptask.getVariable(ptask.getAvailableVariableName());
	m[expr] = 1;			p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
	size_t t = ptask.getVariable(ptask.getAvailableVariableName());
	p = Polynom<GMPint, grevlexC>();
	m[expr] = 1;	m[t] = 1;	p.addNom(m, 1);
	p.addNom(Monom(), -1);
	ptask.addPolynomialCondition(p);
}

void CollinearPointsCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t x1 = ptask.addVariable(myFirst.getX()), y1 = ptask.addVariable(myFirst.getY());
	size_t x2 = ptask.addVariable(mySecond.getX()), y2 = ptask.addVariable(mySecond.getY());
	size_t x3 = ptask.addVariable(myThird.getX()), y3 = ptask.addVariable(myThird.getY());
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[x1] = 1;	m[y3] = 1;	p.addNom(m, 1);		m = Monom();
	m[x1] = 1;	m[y2] = 1;	p.addNom(m, -1);	m = Monom();
	m[x2] = 1;	m[y1] = 1;	p.addNom(m, 1);		m = Monom();
	m[x2] = 1;	m[y3] = 1;	p.addNom(m, -1);	m = Monom();
	m[x3] = 1;	m[y2] = 1;	p.addNom(m, 1);		m = Monom();
	m[x3] = 1;	m[y1] = 1;	p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
}

void PointInSegmentCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t x1 = ptask.addVariable(myA.getX()), y1 = ptask.addVariable(myA.getY());
	size_t x2 = ptask.addVariable(myB.getX()), y2 = ptask.addVariable(myB.getY());
	size_t x = ptask.addVariable(myP.getX()), y = ptask.addVariable(myP.getY());
	size_t t = ptask.getVariable(ptask.getAvailableVariableName());
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[x] = 1;			p.addNom(m, -1);	m = Monom();
	m[x1] = 1;			p.addNom(m, 1);		m = Monom();
	m[x2] = 1;	m[t] = 1;	p.addNom(m, 1);		m = Monom();
	m[x1] = 1;	m[t] = 1;	p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
	p = Polynom<GMPint, grevlexC>();
	m[y] = 1;			p.addNom(m, -1);	m = Monom();
	m[y1] = 1;			p.addNom(m, 1);		m = Monom();
	m[y2] = 1;	m[t] = 1;	p.addNom(m, 1);		m = Monom();
	m[y1] = 1;	m[t] = 1;	p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
}

void DistanceCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t x1 = ptask.addVariable(myFirst.getX()), x2 = ptask.addVariable(mySecond.getX());
	size_t y1 = ptask.addVariable(myFirst.getY()), y2 = ptask.addVariable(mySecond.getY());
	size_t dist = ptask.getVariable(myDistanceVariable);
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, dist));
}

void DistanceToSegmentCondition::convertToPolynoms(PolynomTask &ptask) const {
	size_t xp = ptask.addVariable(myPoint.getX()), yp = ptask.addVariable(myPoint.getY());
	size_t x1 = ptask.addVariable(myFirst.getX()), x2 = ptask.addVariable(mySecond.getX());
	size_t y1 = ptask.addVariable(myFirst.getY()), y2 = ptask.addVariable(mySecond.getY());
	size_t dist = ptask.getVariable(myDistanceVariable);
	size_t t = ptask.addVariable(ptask.getAvailableVariableName());
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, t));
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[y1] = 1;	m[xp] = 1;	p.addNom(m, 1);		m = Monom();
	m[y2] = 1;	m[xp] = 1;	p.addNom(m, -1);	m = Monom();
	m[x2] = 1;	m[yp] = 1;	p.addNom(m, 1);		m = Monom();
	m[x1] = 1;	m[yp] = 1;	p.addNom(m, -1);	m = Monom();
	m[x1] = 1;	m[y2] = 1;	p.addNom(m, 1);		m = Monom();
	m[x2] = 1;	m[y1] = 1;	p.addNom(m, -1);	m = Monom();
	m[t] = 1;	m[dist] = 1;	p.addNom(m, -1);
	ptask.addPolynomialCondition(p);
}

void MiddlePointCondition::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;

	m[ptask.addVariable(myFirst.getX())] = 1;	p.addNom(m, 1);		m = Monom();
	m[ptask.addVariable(mySecond.getX())] = 1;	p.addNom(m, 1);		m = Monom();
	m[ptask.addVariable(myMiddle.getX())] = 1;	p.addNom(m, -2);	m = Monom();
	
	ptask.addPolynomialCondition(p);
	p = Polynom<GMPint, grevlexC>();
	
	m[ptask.addVariable(myFirst.getY())] = 1;	p.addNom(m, 1);		m = Monom();
	m[ptask.addVariable(mySecond.getY())] = 1;	p.addNom(m, 1);		m = Monom();
	m[ptask.addVariable(myMiddle.getY())] = 1;	p.addNom(m, -2);
	
	ptask.addPolynomialCondition(p);
}

void AngleCondition::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t x1 = ptask.addVariable(myLeft.getX()), x2 = ptask.addVariable(myVertex.getX()), x3 = ptask.addVariable(myRight.getX());
	size_t y1 = ptask.addVariable(myLeft.getY()), y2 = ptask.addVariable(myVertex.getY()), y3 = ptask.addVariable(myRight.getY());
	size_t a = ptask.getVariable(ptask.getAvailableVariableName()), b = ptask.getVariable(ptask.getAvailableVariableName()), c = ptask.getVariable(ptask.getAvailableVariableName());

	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x3, y3, a));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, b));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x2, y2, x3, y3, c));

	m[a] = 2;									p.addNom(m, -1);	m = Monom();
	m[b] = 2;		 							p.addNom(m, 1);		m = Monom();
	m[c] = 2;									p.addNom(m, 1);		m = Monom();
	m[a] = 1;	m[b] = 1;	m[ptask.getVariable(myAngleVariable)] = 1;	p.addNom(m, -2);

	ptask.addPolynomialCondition(p);
}

void TriangleAreaCondition::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t a = ptask.addVariable(ptask.getAvailableVariableName()), b = ptask.addVariable(ptask.getAvailableVariableName()), c = ptask.addVariable(ptask.getAvailableVariableName());
	
	size_t x1 = ptask.addVariable(myFirst.getX()), x2 = ptask.addVariable(mySecond.getX()), x3 = ptask.addVariable(myThird.getX());
	size_t y1 = ptask.addVariable(myFirst.getY()), y2 = ptask.addVariable(mySecond.getY()), y3 = ptask.addVariable(myThird.getY());
	
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, a));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x2, y2, x3, y3, b));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x3, y3, c));

	m[a] = 4;			p.addNom(m, -1);	m = Monom();
	m[b] = 4;			p.addNom(m, -1);	m = Monom();
	m[c] = 4;			p.addNom(m, -1);	m = Monom();
	m[a] = 2;	m[b] = 2;	p.addNom(m, 2);		m = Monom();
	m[b] = 2;	m[c] = 2;	p.addNom(m, 2);		m = Monom();
	m[a] = 2;	m[c] = 2;	p.addNom(m, 2);		m = Monom();
	m[ptask.getVariable(myAreaVariable)] = 2;		p.addNom(m, - 16);
	ptask.addPolynomialCondition(p);
}

void EqualCondition::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t left = ptask.getVariable(myLeftVariable);
	m[left] = 1;
	p.addNom(m, 1);
	int t = atoi(myRightVariable.c_str());
	if (t != 0) {
		p.addNom(Monom(), -t);
	} else if (myRightVariable != "0") {
		size_t right = ptask.getVariable(myRightVariable);
		m = Monom();
		m[right] = 1;
		p.addNom(m, -1);
	}
	ptask.addPolynomialCondition(p);
}

void TimesCondition::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t left = ptask.getVariable(myLeftVariable);
	m[left] = 1;
	int t = atoi(myRightVariable.c_str());
	if (t != 0) {
		p.addNom(m, t);
	} else if (myRightVariable != "0") {
		size_t right = ptask.getVariable(myRightVariable);
		m[right] = 1;
		p.addNom(m, 1);
	}
	size_t result = ptask.getVariable(myResultVariable);
	m = Monom();	m[result] = 1;	p.addNom(m, -1);
	ptask.addPolynomialCondition(p);
}

void DistanceQuestion::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t x1 = ptask.addVariable(myFirst.getX()), x2 = ptask.addVariable(mySecond.getX());
	size_t y1 = ptask.addVariable(myFirst.getY()), y2 = ptask.addVariable(mySecond.getY());

	m[x1] = 2;			p.addNom(m, 1);		m = Monom();	
	m[y1] = 2;			p.addNom(m, 1);		m = Monom();
	m[x2] = 2;			p.addNom(m, 1);		m = Monom();
	m[y2] = 2;			p.addNom(m, 1);		m = Monom();
	m[x1] = 1;	m[x2] = 1;	p.addNom(m, -2);	m = Monom();
	m[y1] = 1;	m[y2] = 1;	p.addNom(m, -2);
	
	ptask.setPolynomialQuestion(p);
}

void AngleQuestion::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t x1 = ptask.addVariable(myLeft.getX()), x2 = ptask.addVariable(myVertex.getX()), x3 = ptask.addVariable(myRight.getX());
	size_t y1 = ptask.addVariable(myLeft.getY()), y2 = ptask.addVariable(myVertex.getY()), y3 = ptask.addVariable(myRight.getY());
	size_t a = ptask.addVariable(ptask.getAvailableVariableName()), b = ptask.addVariable(ptask.getAvailableVariableName()), c = ptask.addVariable(ptask.getAvailableVariableName());

	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x3, y3, a));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, b));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x2, y2, x3, y3, c));

	size_t cosa = ptask.addVariable(ptask.getAvailableVariableName());

	m[a] = 2;					p.addNom(m, -1);	m = Monom();
	m[b] = 2; 					p.addNom(m, 1);		m = Monom();
	m[c] = 2;					p.addNom(m, 1);		m = Monom();
	m[a] = 1;	m[b] = 1;	m[cosa] = 1;	p.addNom(m, -2);	m = Monom();

	ptask.addPolynomialCondition(p);	p = Polynom<GMPint, grevlexC>();
	
	m[cosa] = 1;					p.addNom(m, 1);
	
	ptask.setPolynomialQuestion(p);
}

void TriangleAreaQuestion::convertToPolynoms(PolynomTask &ptask) const {
	Monom m;
	Polynom<GMPint, grevlexC> p;
	size_t a = ptask.addVariable(ptask.getAvailableVariableName()), b = ptask.addVariable(ptask.getAvailableVariableName()), c = ptask.addVariable(ptask.getAvailableVariableName());
	
	size_t x1 = ptask.addVariable(myFirst.getX()), x2 = ptask.addVariable(mySecond.getX()), x3 = ptask.addVariable(myThird.getX());
	size_t y1 = ptask.addVariable(myFirst.getY()), y2 = ptask.addVariable(mySecond.getY()), y3 = ptask.addVariable(myThird.getY());
	
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, a));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x2, y2, x3, y3, b));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x3, y3, c));
	
	m[a] = 4;			p.addNom(m, -1);	m = Monom();
	m[b] = 4;			p.addNom(m, -1);	m = Monom();
	m[c] = 4;			p.addNom(m, -1);	m = Monom();
	m[a] = 2;	m[b] = 2;	p.addNom(m, 2);		m = Monom();
	m[b] = 2;	m[c] = 2;	p.addNom(m, 2);		m = Monom();
	m[a] = 2;	m[c] = 2;	p.addNom(m, 2);		m = Monom();
	ptask.setPolynomialQuestion(p);
}

void TrapeziumAreaQuestion::convertToPolynoms(PolynomTask &ptask) const {
	size_t a = ptask.getVariable(ptask.getAvailableVariableName());
	size_t b = ptask.getVariable(ptask.getAvailableVariableName());
	size_t c = ptask.getVariable(ptask.getAvailableVariableName());
	size_t d = ptask.getVariable(ptask.getAvailableVariableName());
	size_t  x1 = ptask.addVariable(myA.getX()), y1 = ptask.addVariable(myA.getY());
	size_t  x2 = ptask.addVariable(myB.getX()), y2 = ptask.addVariable(myB.getY());
	size_t  x3 = ptask.addVariable(myC.getX()), y3 = ptask.addVariable(myC.getY());
	size_t  x4 = ptask.addVariable(myD.getX()), y4 = ptask.addVariable(myD.getY());
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x2, y2, a));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x3, y3, x4, y4, b));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x1, y1, x3, y3, c));
	ptask.addPolynomialCondition(ptask.makeDistancePolynom(x2, y2, x4, y4, d));
	size_t h = ptask.getVariable(ptask.getAvailableVariableName());
	size_t f = ptask.getVariable(ptask.getAvailableVariableName());
	Monom m;
	Polynom<GMPint, grevlexC> p;
	m[a] = 2;			p.addNom(m, 1);		m = Monom();
	m[b] = 2;			p.addNom(m, 1);		m = Monom();
	m[c] = 2;			p.addNom(m, -1);	m = Monom();
	m[d] = 2;			p.addNom(m, 1);		m = Monom();
	m[a] = 1;	m[b] = 1;	p.addNom(m, -2);	m = Monom();
	m[a] = 1;	m[f] = 1;	p.addNom(m, 2);		m = Monom();
	m[b] = 1;	m[f] = 1;	p.addNom(m, -2);	m = Monom();
	ptask.addPolynomialCondition(p);
	p = Polynom<GMPint, grevlexC>();
	m[h] = 2;			p.addNom(m, -1);	m = Monom();
	m[d] = 2;			p.addNom(m, 1);		m = Monom();
	m[f] = 2;			p.addNom(m, -1);	m = Monom();
	ptask.addPolynomialCondition(p);
	p = Polynom<GMPint, grevlexC>();
	m[a] = 1;	m[h] = 1;	p.addNom(m, 1);		m = Monom();
	m[b] = 1;	m[h] = 1;	p.addNom(m, 1);		m = Monom();
	ptask.setPolynomialQuestion(p);
}

void TriangleAreaQuestion::postProcessSolution(GMPint &numerator, GMPint &denominator) {
	numerator /= 16;
}

void TrapeziumAreaQuestion::postProcessSolution(GMPint &numerator, GMPint &denominator) {
}

void DistanceQuestion::postProcessSolution(GMPint &numerator, GMPint &denominator) {
	//numerator = (long int)sqrt(numerator.getlongint());
	//denominator = (long int)sqrt(denominator.getlongint());
}

void AngleQuestion::postProcessSolution(GMPint &numerator, GMPint &denominator) {
}
