#include "gmpint.h"
#include "tokenizer.h"
#include "syntaxtree.h"
#include "generator.h"
#include "parser.h"
#include "task.h"
#include "nodehandlers.h"
#include "point.h"

Generator::Generator(Parser &parser) {
	parser.parse(myTrees);

	myNodeHandlers["Middle"] = new MiddlePointNodeHandler();
	myNodeHandlers["Angle"] = new AngleNodeHandler();
	myNodeHandlers["Distance"] = new DistanceNodeHandler();
	myNodeHandlers["Triangle"] = new TriangleNodeHandler();
	myNodeHandlers["TriangleArea"] = new TriangleAreaNodeHandler();
	myNodeHandlers["Equal"] = new EqualNodeHandler();
	myNodeHandlers["Times"] = new TimesNodeHandler();
	myNodeHandlers["FindDistance"] = new FindDistanceNodeHandler();
	myNodeHandlers["FindAngle"] = new FindAngleNodeHandler();
	myNodeHandlers["FindTriangleArea"] = new FindTriangleAreaNodeHandler();
	myNodeHandlers["Cos"] = new CosNodeHandler();
	myNodeHandlers["ThreeCollinearPoints"] = new CollinearPointsNodeHandler();
	myNodeHandlers["FindTrapeziumArea"] = new FindTrapeziumAreaNodeHandler();
	myNodeHandlers["PointInSegment"] = new PointInSegmentNodeHandler();
	myNodeHandlers["ParallelLines"] = new ParallelLinesNodeHandler();
	myNodeHandlers["PointDivideSegment"] = new PointDivideSegmentNodeHandler();
}

Generator::~Generator() {
	for (std::vector<SyntaxTreeNode *>::iterator i = myTrees.begin(); i != myTrees.end(); ++i) {
		(*i)->deleteTree();
	}
}

void Generator::addStatementToTask(Task &task, SyntaxTreeNode *node) {
	if (node != 0 && node->hasChildren()) {
		for (size_t i = 0; i != node->childrenNumber(); ++i) {
			addStatementToTask(task, node->getChild(i));
		}
		std::map<std::string, Token>::iterator it = myHashTable.find(node->hash());
		if (it != myHashTable.end()) {
			node->deleteSubTrees();
			node->setToken(it->second);
		} else {
			std::string value = node->getToken().getValue();
			std::map<std::string, NodeHandler *>::const_iterator i = myNodeHandlers.find(value);
			if (i == myNodeHandlers.end()) {
				std::cerr << "Not found handler for " << value << std::endl;
				return;
			}
			NodeHandler *currentHandler = i->second;
			if (currentHandler->isValid(node)) {
				Token token = currentHandler->addToModel(task, node);
				myHashTable.insert(std::make_pair(node->hash(), token));
				node->deleteSubTrees();
				node->setToken(token);
			} else {
				std::cerr << "Invalide signature of statement" << std::endl;
			}
		}
	}
}

Task Generator::generate() {
	Task result;
	for (std::vector<SyntaxTreeNode *>::const_iterator i = myTrees.begin(); i != myTrees.end(); ++i) {
		addStatementToTask(result, *i);
	}
	return result;
}
