#include <sstream>

#include "gmpint.h"
#include "Polynom.h"
#include "Polynom.impl.h"
#include "polynomtask.h"

PolynomTask::PolynomTask() : myAvailableVariableNamePrefix("TemporaryVariable") {
	myAvailableVariableNumber = 0;
}

void PolynomTask::addPolynomialCondition(Polynom<GMPint, grevlexC> condition) {
	myBasis.addPolynom(condition);
}

void PolynomTask::setPolynomialQuestion(Polynom<GMPint, grevlexC> question) {
	myQuestion = question;
}

size_t PolynomTask::addVariable(const std::string &var) {
	return myVars.addVar(var);
}

size_t PolynomTask::getVariable(const std::string &var) {
	return myVars.getNum(var);
}

std::pair<GMPint, GMPint> PolynomTask::getValue() {
	GMPint n, d;
	if (myQuestion.getValue(n, d, myBasis) == 0) {
		return std::make_pair(0, 0);
	}
	return std::make_pair(n, d);
}

std::string PolynomTask::getAvailableVariableName() {
	while (myVars.getNum(getCurrentAvailableVariableName()) != (size_t)-1) {
		++myAvailableVariableNumber;
	}
	myVars.addVar(getCurrentAvailableVariableName());
	return getCurrentAvailableVariableName();
}

std::string PolynomTask::getCurrentAvailableVariableName() const {
	std::ostringstream oss;
	oss << myAvailableVariableNamePrefix << myAvailableVariableNumber;
	return oss.str();
}

Polynom<GMPint, grevlexC> PolynomTask::makePolynom(const std::string &poly) {
	return myVars.strtoPolynom<GMPint, grevlexC>(poly);
}


Polynom<GMPint, grevlexC> PolynomTask::makeDistancePolynom(size_t x1, size_t y1, size_t x2, size_t y2, size_t dist) {
	Monom m;
	Polynom<GMPint, grevlexC> p;

	m[x1] = 2;			p.addNom(m, 1);		m = Monom();
	m[y1] = 2;			p.addNom(m, 1);		m = Monom();
	m[x2] = 2;			p.addNom(m, 1);		m = Monom();
	m[y2] = 2;			p.addNom(m, 1);		m = Monom();
	m[x1] = 1;	m[x2] = 1;	p.addNom(m, -2);	m = Monom();
	m[y1] = 1;	m[y2] = 1;	p.addNom(m, -2);	m = Monom();
	m[dist] = 2;			p.addNom(m, -1);
	
	return p;
}
