#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include "gmpint.h"
#include "task.h"
#include "tokenizer.h"
#include "parser.h"
#include "generator.h"
#include "point.h"

class Solver {

public:
	Solver(const std::string &filename);
	~Solver();

	std::vector<std::pair<GMPint, GMPint> > solve();

private:
	std::ifstream myFile;

};

int main(int argc, char *argv[]) {
	if (argc < 2) {
		std::cout << "Usage: ./solver filename" << std::endl;
		return 1;
	}

	std::vector<std::pair<GMPint, GMPint> > answers = Solver(argv[1]).solve();
	size_t size = answers.size();
	for (size_t i = 0; i < size; ++i) {
		GMPint den = answers[i].second;
		if (den == 0) {
			std::cout << "The problem has not solution" << std::endl;
		} else {
			std::cout << answers[i].first;
			if (den != 1) {
				std::cout << '/' << den;
			}
			std::cout << std::endl;
		}
	}
	
	return 0;
}

Solver::Solver(const std::string &filename) {
	myFile.open(filename.c_str());
}

Solver::~Solver() {
	myFile.close();
}

std::vector<std::pair<GMPint, GMPint> > Solver::solve() {
	Tokenizer t(myFile);
	Parser p(t);
	Generator g(p);
	Task task = g.generate();
	return task.solve();
}
