#include <algorithm>
#include "tokenizer.h"
#include "syntaxtree.h"

void SyntaxTreeNode::deleteTree() {
	for (std::vector<SyntaxTreeNode *>::iterator i = myChildren.begin(); i != myChildren.end(); ++i) {
		(*i)->deleteTree();
	}
	delete this;
}

void SyntaxTreeNode::deleteSubTrees() {
	for (std::vector<SyntaxTreeNode *>::iterator i = myChildren.begin(); i != myChildren.end(); ++i) {
		(*i)->deleteTree();
	}
	myChildren.clear();
}

SyntaxTreeNode::SyntaxTreeNode(const Token &token) : myToken(token) {
}

void SyntaxTreeNode::dump(std::ostream &out) const {
	out << "Node: " << myToken.getValue() << std::endl;
	if (!(myChildren.empty())) {
		out << "Children: " << std::endl;
	}
	for (std::vector<SyntaxTreeNode *>::const_iterator i = myChildren.begin(); i != myChildren.end(); ++i) {
		(*i)->dump(out);
	}
}

std::ostream &operator<<(std::ostream &out, const SyntaxTreeNode &node) {
	node.dump(out);
	return out;
}

bool SyntaxTreeNode::tokenLess(SyntaxTreeNode *node1, SyntaxTreeNode *node2) {
	return node1->getToken().getValue() < node2->getToken().getValue();
}

std::string SyntaxTreeNode::hash() {
	std::vector<SyntaxTreeNode *> children;
	std::copy(myChildren.begin(), myChildren.end(), std::back_inserter(children));
	std::sort(children.begin(), children.end(), tokenLess);
	std::string code = myToken.getValue();
	for (std::vector<SyntaxTreeNode *>::const_iterator i = children.begin(); i != children.end(); ++i) {
		code += (*i)->hash();
	}
	return code;
}

void SyntaxTreeNode::addChild(SyntaxTreeNode *child) {
	myChildren.push_back(child);
}

bool SyntaxTreeNode::hasChildren() const {
	return !myChildren.empty();
}

size_t SyntaxTreeNode::childrenNumber() const {
	return myChildren.size();
}

SyntaxTreeNode *SyntaxTreeNode::getChild(size_t i) const {
	return myChildren[i];
}

void SyntaxTreeNode::setToken(const Token &token) {
	myToken = token;
}

Token SyntaxTreeNode::getToken() const {
	return myToken;
}
