#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <vector>
#include <dirent.h>

static bool equalsFiles(std::string file1, std::string file2);
void scanDir(const std::string &path, std::vector<std::string> &files);

int main(int argc, char *argv[]) {
	std::vector<std::string> filenames;
	scanDir("in/", filenames);
	std::string realOutDir("realOut/");
	bool firstGenerate = false;
	if (argc > 1) {
		realOutDir = argv[1];
		firstGenerate = true;
	}
	std::string inDir("in/");
	std::string outDir("out/");
	std::string exec, output;
	bool done = true;
	for (size_t i = 0; i < filenames.size(); ++i) {
		std::string input = inDir + filenames[i] + ".geo";
		output = std::string("") + "tok" + filenames[i] + ".out";
		exec = "./testTokenizer " + input + " " +  realOutDir + output;
		system(exec.c_str());
		if (!firstGenerate && !equalsFiles(outDir + output, realOutDir + output)) {
			std::cerr << "Tokenizer didn't pass " << input << std::endl;
			done = false;
		}

		output = std::string("") + "par" + filenames[i] + ".out";
		exec = "./testParser " + input + " " + realOutDir + output;
		system(exec.c_str());
		if (!firstGenerate && !equalsFiles(outDir + output, realOutDir + output)) {
			std::cerr << "Parser didn't pass " << input << std::endl;
			done = false;
		}
		
		output = std::string("") + "gen" + filenames[i] + ".out";
		exec = "./testGenerator " + input + " " + realOutDir + output;
		system(exec.c_str());
		if (!firstGenerate && !equalsFiles(outDir + output, realOutDir + output)) {
			std::cerr << "Generator didn't pass " << input << std::endl;
			done = false;
		}
	}
	if (firstGenerate) {
		std::cout << "Test's output was generated" << std::endl;
		return 0;
	}
	if (done) {
		std::cout << "All tests complete successfull" << std::endl;
	}
}

bool equalsFiles(std::string file1, std::string file2) {
	std::ifstream in1(file1.c_str());
	std::ifstream in2(file2.c_str());
	in1.seekg(0, std::ios::end);
	in2.seekg(0, std::ios::end);
	int size1 = in1.tellg(), size2 = in2.tellg();
	in1.seekg(0, std::ios::beg);
	in2.seekg(0, std::ios::beg);
	if (size1 != size2) {
		in1.close();
		in2.close();
		return false;
	}
	for (int i = 0; i < size1; i++) {
		if (in1.get() != in2.get()) {
			in1.close();
			in2.close();
			return false;
		}
	}
	in1.close();
	in2.close();
	return true;
}

void scanDir(const std::string &path, std::vector<std::string> &files) {
	DIR *dir = opendir(path.c_str());
	dirent *dirEntry;
	while ((dirEntry = readdir(dir)) != 0) {
		std::string fname(dirEntry->d_name);
		std::string ext(".geo");
		if (fname.length() > ext.length()) {
			if (fname.substr(fname.length() - ext.length(), ext.length()) == ext) {
				files.push_back(fname.substr(0, fname.length() - ext.length()));
			}
		}
	}
	closedir(dir);
}
